/*
 *  ident.c - TCP/IP ident protocol query
 *
 *  Copyright (C) 1998 Hugo Haas
 *  Based on cfingerd implementation, written by Joel Katz - see copyright below
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * CFINGERD
 * RFC1413 implementation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <signal.h>
#include <unistd.h>
#include <netdb.h>
#include <string.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#include "defines.h"
#include "ident.h"

#define TIMEOUT 5

void timeout_alarm(int signal) {
}

void get_ident_info(char *username, unsigned short int local_port, unsigned short int remote_port, __u32 remote_addr) {
  int s;
  struct sockaddr_in sin;
  char buffer[LINE], uname[64];

  bzero(uname, USERNAME_LENGTH);

  s = socket(AF_INET, SOCK_STREAM, 0);
  if (s < 0)
    return;

  sin.sin_family = AF_INET;
  sin.sin_port = TCP_IDENT_PORT;
  sin.sin_addr.s_addr = remote_addr;

  signal(SIGALRM, timeout_alarm);
  alarm(TIMEOUT);

  if (connect(s, (struct sockaddr_in *) &sin, sizeof(sin)) < 0) {
    close(s);
    alarm(0);
    return;
  }
  sprintf(buffer, "%d,%d\n", local_port, remote_port);
  write(s, buffer, strlen(buffer));

  if (read(s, buffer, LINE) > 0) {
    sscanf(buffer, "%*d , %*d : %*[^ \t\n\r:] : %*[^\t\n\r:] : %[^\n\r]", uname);
    printf(buffer);
    strcpy(username, uname);
  }

  alarm(0);
  close(s);
  return;
}

/*
 * $Log: ident.c,v $
 * Revision 1.1  1998/10/27 23:58:12  hugo
 * Initial revision
 *
 */
